/* Text Printer class  *\
\* By Justin Pease     */

#pragma once

#include "gamex.hpp"

class TextPrinter {
public:
	// Initialize text printer (by loading text graphics)
	// if you give charListInImageOrder, it should be a (null-terminated) string of characters in the top-to-bottom, left-to-right order they're in the image, otherwise a default order is assumed
	void Initialize(ImageX * alphabetImage, int charsAcrossImage=16);
	void Initialize(ImageX * alphabetImage, int charsAcrossImage, const char * charListInImage);

	// Prints out a string of text graphically using the text alphabet image text_img
	// - fontSize is the point size of the font (dimensions in pixels of each character - may need to set this larger than you'd think necessary if your text image has much spacing around the letters)
	// - drawMode is a GameX draw mode such as DRAW_GHOST or DRAW_ADD or DRAW_SUBTRACT|DRAWOP_NOFILTER etc.
	// - if rotateBaseline is false, angle will rotate all the individual letters instead of the whole string
	// - if fixedWidth is true, the text is printed like a fixed-width font (but maybe too wide, should fix)
	void PrintText(const char * str, float x, float y, float fontSize=24, bool centered=false, int drawMode=DRAW_PLAIN, ColorX & color=ColorX(), float angle=0.0f, float yScale=1.0f, bool rotateBaseline=true, bool fixedWidth = false);
	float GetVisibleCharWidth(const char * str, int index, float font_size);
	float GetVisibleStringWidth(const char * str, int first_index, int last_index, float font_size);
	float GetVisibleStringWidth(const char * str, float font_size);
	void GetVisibleCharInfo(const char * str, int n, int & xs, int & ys, float & kern, bool & visible);

	void PrintTextScreenProportional(const char * str, float x, float y, float heightRatio, float xStretch=1.0f, bool centered=false, int draw_mode=DRAW_PLAIN, ColorX & color=ColorX(), float angle=0.0f, bool rotate_baseline=true, bool fixed = false);
	float GetVisibleStringWidthProportional(const char * str, float heightRatio, float xStretch=1.0f);
	float GetVisibleStringWidthProportional(const char * str, int first_index, int last_index, float heightRatio, float xStretch=1.0f);

	inline void SetHighlight(ColorX c, char startChar, char endChar) {highlightColor = c; highlightStartChar = startChar; highlightEndChar = endChar;}

	inline bool IsInitialized() {return (text_img != NULL);}

private:
	void GetCharOffsets(char c, int & xs, int & ys); // called only at initialize time (hopefully)

	ImageX * text_img;

	int charPixels;
	float fCharPixels;

	const static int HEIGHT_LEVELS = 12;
	struct charInfo
	{
		short xs, ys; // top-left coordinate where the character can be found in the text image
		unsigned char xl [HEIGHT_LEVELS], xr [HEIGHT_LEVELS]; // left and right max extents at given height interval of each character, normalized from 0 to 256
		bool visible;
	};
	charInfo chars [256];

	ColorX highlightColor;
	char highlightStartChar, highlightEndChar;
	bool inHighlight;
	float _avgTop, _avgBot; /// TEMP put in class
};

extern TextPrinter tp; // tp is a global TextPrinter object
